/*
Copyright (c) 1999, Braden N. McDaniel

Permission to use, copy, modify, distribute and sell this software and its
documentation for any purpose is hereby granted without fee, provided that the
copyright notice appear in all copies and that both that copyright notice and
this permission notice appear in supporting documentation. Braden N. McDaniel
makes no representations about the suitability of this software for any
purpose. It is provided "as is" without express or implied warranty.
*/

import vrml.*;
import vrml.field.*;
import vrml.node.*;

public class ColorArrayInterpolator extends Script {

    public void initialize() {
        this.key         = (MFFloat)getField("key");
        this.keyValue    = (MFColor)getField("keyValue");
        this.validValues = (SFBool)getField("validValues");

        this.key_changed      = (MFFloat)getEventOut("key_changed");
        this.keyValue_changed = (MFColor)getEventOut("keyValue_changed");
        this.value_changed    = (MFColor)getEventOut("value_changed");

        this.checkValues();
    }

    public void processEvent(Event event) {
        if (event.getName().equals("set_fraction")) {

            MFColor colors = new MFColor();

            float fraction = ((ConstSFFloat)event.getValue()).getValue();

            if (validValues.getValue()) {

                int arraySize = (keyValue.getSize() / key.getSize());
                int keyMinIndex = getKeyMinIndex(fraction);
                int valueStartIndex = (keyMinIndex * arraySize);
                float intermediateFraction = getIntermediateFraction(keyMinIndex, fraction);

                if (fraction == this.key.get1Value(keyMinIndex)) {
                    for (int i = 0, j = valueStartIndex; i < arraySize; i++, j++) {
                        SFColor color = new SFColor();
                        keyValue.get1Value(j, color);
                        colors.addValue(color);
                    }
                } else {
                    for (int i = 0, j = valueStartIndex; i < arraySize; i++, j++) {
                        SFColor color1 = new SFColor(), color2 = new SFColor();
                        keyValue.get1Value(j, color1);
                        keyValue.get1Value(j + arraySize, color2);
                        colors.addValue(interpolateColor(color1, color2, intermediateFraction));
                    }
                }
            } else {
                System.err.println("ColorArrayInterpolator has invalid values.");
            }

            this.value_changed.setValue(colors);

        } else if (event.getName().equals("set_key")) {
            this.key.setValue((ConstMFFloat)event.getValue());
            this.key_changed.setValue((ConstMFFloat)event.getValue());
        } else if (event.getName().equals("set_keyValue")) {
            this.keyValue.setValue((ConstMFColor)event.getValue());
            this.keyValue_changed.setValue((ConstMFColor)event.getValue());
        }
    }

    private void checkValues() {
        if ((key.getSize() >= 2) && ((keyValue.getSize() % key.getSize()) == 0)) {
            this.validValues.setValue(true);
        } else {
            this.validValues.setValue(false);
        }
    }

    private int getKeyMinIndex(float fraction) {
        int index = 0;
        while (((index + 1) < key.getSize()) && (key.get1Value(index + 1) < fraction)) {
            index++;
        }
        return index;
    }

    private float getIntermediateFraction(int keyMinIndex, float fraction) {
        float intervalDifference = (this.key.get1Value(keyMinIndex + 1) - this.key.get1Value(keyMinIndex));
        float intervalElapsed = (fraction - this.key.get1Value(keyMinIndex));
        return (intervalElapsed / intervalDifference);
    }

    private SFColor interpolateColor(SFColor sfcolor1, SFColor sfcolor2, float intermediateFraction) {
        return new SFColor(sfcolor1.getRed() + ((sfcolor2.getRed() - sfcolor1.getRed()) * intermediateFraction),
                           sfcolor1.getGreen() + ((sfcolor2.getGreen() - sfcolor1.getGreen()) * intermediateFraction),
                           sfcolor1.getBlue() + ((sfcolor2.getBlue() - sfcolor1.getBlue()) * intermediateFraction));
    }

    private MFFloat key;
    private MFColor keyValue;
    private SFBool  validValues;

    private MFFloat key_changed;
    private MFColor keyValue_changed;
    private MFColor value_changed;



}
